#include "ablib.h"
#include "BMSearch.h"
#include "IsKanji.h"

#ifdef UNICODE
typedef wchar_t UTCHAR;
#else
typedef unsigned char UTCHAR;
#endif

namespace ablib{


void BMSearch::MakeTable(const TCHAR *str,size_t len,size_t table[]){
	if(len == 1)return;
	for(int i = 0 ; i < CHARS_NUM ; ++i)table[i] = len;
	while(len > 0)table[static_cast<UTCHAR>(*str++)] = --len;
}

intptr_t BMSearch::Search(const TCHAR *target,size_t targetlen,const TCHAR *search, size_t searchlen, size_t table[]){
	--searchlen;
	if(searchlen < 0)return -1;
	if(searchlen == 0){
		for(int i = 0 ; i < targetlen ; ++i){
#ifdef UNICODE
			if(target[i] == *search)return i;
#else
			if(target[i] == *search && !IsKanji2(target,i))return i;
#endif
		}
		return -1;
	}
	const TCHAR *end = target + targetlen - searchlen;
	const TCHAR *start = target;
	intptr_t i = 0;
	intptr_t j;
	while(target < end){
		for(i = searchlen ; i >= 0 ; --i)if(target[i] != search[i])break;
#ifdef UNICODE
		if(i < 0)return target - start;
		else j = table[static_cast<UTCHAR>(target[i])] - (searchlen - i);
#else
		if(i < 0){
			if(!IsKanji2(start,target - start))return target - start;
			else j = 1;
		}else j = table[static_cast<UTCHAR>(target[i])] - (searchlen - i);
#endif
		target += j > 0 ? j : 2;
	}
	return -1;
}

void BMRSearch::MakeTable(const TCHAR *str,size_t len,size_t table[]){
	if(len <= 1)return;
	for(int i = 0 ; i < CHARS_NUM ; ++i)table[i] = len;
	while(--len >= 0)table[static_cast<UTCHAR>(*(str + len))] = len;
}

intptr_t BMRSearch::Search(const TCHAR *target,size_t targetlen,const TCHAR *search,size_t searchlen,size_t table[]){
	--searchlen;
	if(searchlen < 0)return -1;
	else if(searchlen == 0){
		for(intptr_t i = targetlen - 1 ; i >= 0 ; --i){
#ifdef UNICODE
			if(target[i] == *search)return i;
#else
			if(target[i] == *search && !IsKanji2(target,i))return i;
#endif
		}
		return -1;
	}

	const TCHAR *start = target;
	target += targetlen - searchlen;
	size_t i = 0;
	intptr_t j;
	while(target >= start){
		for(i = 0 ; i <= searchlen ; ++i)if(target[i] != search[i])break;
#ifdef UNICODE
		if(i > searchlen)return target - start;
		else j = table[static_cast<UTCHAR>(target[i])] - i;
#else
		if(i > searchlen){
			if(!IsKanji2(start,target - start))return target - start;
			else j = 1;
		}else j = table[static_cast<UTCHAR>(target[i])] - i;
#endif
		target -= j > 0 ? j : 2;
	}
	return -1;
}

}//namespace ablib

