#ifndef __ABLIB_TEXTFILE_H__
#define __ABLIB_TEXTFILE_H__

#include "ablib.h"
#include <stdio.h>

namespace ablib{

enum charset{
	AutoDetect = 0,
	Western    = -1,
	Turkish    = 1254,
	Hebrew     = 1255,
	Arabic     = 1256,
	Baltic     = 1257,
	Vietnamese = 1258,
	Central    = 28592,
	Greek      = 28597,
	Thai       = 874,

	Cyrillic   = 28595,
	Koi8R      = 20866,
	Koi8U      = 21866,

	UHC        = 949,
	IsoKR      = -950,
	Johab      = 1361,

	GBK        = 936,
	IsoCN      = -936,
	HZ         = -937,
	Big5       = 950,

	SJIS       = 932,
	EucJP      = -932,
	IsoJP      = -933,

	UTF5       = -2,
	UTF7       = 65000,
	UTF8       =-65001,
	UTF8N      = 65001,
	UTF16b     = -3,
	UTF16l     = -4,
	UTF16BE    = -5,
	UTF16LE    = -6,
	UTF32b     = -7,
	UTF32l     = -8,
	UTF32BE    = -9,
	UTF32LE    = -10,

	DOSUS      = 437
};

enum lbcode{
	CR   = 0,
	LF   = 1,
	CRLF = 2
};

/** eLXgt@CǂݏNX
* w肳ꂽR[hϊȂǂݍ݂܂DԂR[hɔYނƂ肠Shift_JISŁD
*/
class TextFile{
public:
	typedef size_t size_type;
	enum filemode{read,write,append};
	TextFile() : m_Read(NULL),m_Write(NULL){};
	~TextFile(){Close();}

	bool Open(const TCHAR *file,filemode mode,int cs = AutoDetect,int lb = CRLF);
	bool Open(const ablib::string &file,filemode mode,int cs = AutoDetect,int lb = CRLF){
		return Open(file.c_str(),mode,cs,lb);
	}
	void Close(){
		if(m_Read){delete m_Read;m_Read = NULL;}
		if(m_Write){delete m_Write;m_Write = NULL;}
	}

	int codepage(){return m_cs;}
	int linebreak(){return m_lb;}

	bool IsOpen() const{return (m_Read != NULL || m_Write != NULL);}
	operator bool() const{return IsOpen();}

private:
	int m_cs,m_lb;
	void AutoDetection(BYTE *buf,size_type size,int defcs,int deflb);

//ǂݍ
public:
	size_type ReadLine(TCHAR *buf,size_type size){ASSERT(size >= 16);if(m_Read)return m_Read->ReadLine(buf,size);else return 0;}
	size_type Read(TCHAR *buf,size_type size){ASSERT(size >= 16);if(m_Read)return m_Read->Read(buf,size);else return 0;}

private:
	struct READ{
		virtual ~READ(){};
		virtual size_type ReadLine(TCHAR *buf,size_type size) = 0;
		virtual size_type Read(TCHAR *buf,size_type size) = 0;
	}*m_Read;

	struct rEUC;
	struct rIsoJP;
	struct rSJIS;
	struct rDirect;

	struct rUTF32;
	struct rUTF16;
	struct rUTF7_8;
	struct rUTF5;


//
public:
	size_type Write(const TCHAR *buf,size_type size){if(m_Write)return m_Write->Write(buf,size);else return 0;}
	size_type Write(const TCHAR *buf){return Write(buf,::lstrlen(buf));}
	size_type Write(const string &buf){return Write(buf.c_str(),buf.length());}

private:
	struct WRITE{
		virtual ~WRITE(){};
		virtual size_type Write(const TCHAR *buf,size_type size) = 0;
	}*m_Write;

	struct wEUC;
	struct wIsoJP;
	struct wSJIS;
	struct wDirect;
	struct wUTF32;
	struct wUTF16;
	struct wUTF7_8;
	struct wUTF5;

private:
	/** t@CꕶǂݍłxĂ܂ȂiƎvĵłǂƓǂݍłD
	*/
	template<typename T> struct FileRead : public READ{
		static const int TEXTFILE_BUF = 4096;
		FileRead(FILE *fp) : m_fp(fp),m_eof(false),m_nowread(0){
			m_length = ::fread(m_buf,sizeof(T),TEXTFILE_BUF,m_fp);
			if(m_length == 0)m_eof = true;
		}
		virtual ~FileRead(){if(m_fp){::fclose(m_fp);m_fp = NULL;}}

		T m_buf[TEXTFILE_BUF];
		FILE *m_fp;
		bool m_eof;
		enum{eol,eob,eof}m_state;
		size_type m_nowread,m_length;

		void Reset(){
			::fseek(m_fp,0,SEEK_SET);m_eof = false;m_nowread = 0;;
			m_length = ::fread(m_buf,sizeof(T),TEXTFILE_BUF,m_fp);
			if(m_length == 0)m_eof = true;
		}

		bool IsEof()const{return m_eof;}
		T GetChar(){
			if(m_fp == NULL || m_eof)return 0;
			T rv = m_buf[m_nowread++];
			if(m_nowread == m_length){
				m_length = ::fread(m_buf,sizeof(T),TEXTFILE_BUF,m_fp);
				if(m_length == 0)m_eof = true;
				m_nowread = 0;
			}
			return rv;
		}
		T GetOneChar(){
			if(m_fp == NULL || m_eof)return 0;
			return m_buf[m_nowread];
		}
	};

	/** obt@OĂĂǂƏށD
	*/
	template<typename T> struct FileWrite : public WRITE{
		static const int TEXTFILE_BUF = 4096;
		FileWrite(FILE *fp) : m_fp(fp),m_ptr(0){}
		virtual ~FileWrite(){
			if(m_fp){
				::fwrite(m_buf,sizeof(T),m_ptr,m_fp);
				::fclose(m_fp);
			}
		}
		FILE *m_fp;
		size_type m_ptr;
		T m_buf[TEXTFILE_BUF];
		void WriteChar(T c){
			if(m_ptr == TEXTFILE_BUF){
				size_type s = 0;
				for(int i = 0 ; i < 10 && s < TEXTFILE_BUF ; ++i){
					s += ::fwrite(m_buf + s,sizeof(T),TEXTFILE_BUF - s,m_fp);
				}
				m_ptr = 1;
				m_buf[0] = c;
			}else m_buf[m_ptr++] = c;
		}
	};
};

}//namespace ablib

#endif //__ABLIB_TEXTFILE_H__

