#include "edit.h"
#include "memory.h"
#include "struct.h"
#include "../ablib/BMsearch.h"

namespace ablib{
namespace window{
namespace edit{

ablib::string tmp_string;

Memory::Memory(void){
	m_lastpos = m_buf.begin();
	m_buf.push_back(_T(""));
}

Memory::~Memory(void)
{
}

const TCHAR *Memory::GetText(int line){
//	static ablib::string rv;
	iterator ite = SearchLine(line);
	if(ite == m_buf.end())return NULL;
//	rv = ite->c_str();
	m_lastpos = ite;
	m_lastline = line;
	return ite->c_str();
}

const TCHAR *Memory::GetText(POS start,POS end,const TCHAR *sep){
	tmp_string = _T("");
	if(start > end){
		POS tmp = start;
		start = end;
		end = tmp;
	}
	iterator ite = SearchLine(start.lineno);
	if(ite == m_buf.end())return NULL;
	if(start.lineno == end.lineno){
	    tmp_string = ite->substr(start.column,end.column - start.column);
		return tmp_string.c_str();
	}
	tmp_string = ite->substr(start.column);
	tmp_string += sep;
	tmp_string.reserve((end.lineno - start.lineno) * 50);
	for(int i = start.lineno + 1 ; i < end.lineno ; ++i){
		++ite;
		tmp_string += *ite;
		tmp_string += sep;
	}
	++ite;
	tmp_string += ite->substr(0,end.column);
	m_lastpos = ite;
	m_lastline = end.lineno;
	return tmp_string.c_str();
}

bool Memory::InsertText(POS pos,POS &endpos,const TCHAR *str,const TCHAR *sep){
	iterator ite = SearchLine(pos.lineno);
	if(ite == m_buf.end())return false;
	bool needanalyzelen = false;
	endpos = pos;
	ablib::string reststr;
	const TCHAR *p,*q;
	int seplen = ::lstrlen(sep);
	p = ablib::BMSearch::strstr(str,sep);
	if(p == NULL){
		int oldlen = ite->length();
		ite->insert(pos.column,str);
		endpos.column += ::lstrlen(str);
		m_maxlen = Max(GetLayoutLen(*ite),m_maxlen);
		return true;
	}
	reststr.assign(*ite,pos.column,ablib::string::npos);
	if(m_maxlen == ite->length())needanalyzelen = true;
	ite->erase(pos.column);
	ite->append(str,p - str);
	q = p + seplen;
	ablib::string s;
	while(p = ablib::BMSearch::strstr(q,sep)){
		++ite;
		s.assign(q,0,p - q);
		ite = m_buf.insert(ite,s);
		++endpos.lineno;
		q = p + seplen;
	}
	++ite;
	ite = m_buf.insert(ite,q);
	++endpos.lineno;
	endpos.column = ite->length();
	ite->append(reststr);
	m_lastpos = ite;
	m_lastline = endpos.lineno;
	needanalyzelen = true;
	if(needanalyzelen){
		m_maxlen = 0;
		iterator end = m_buf.end();
		for(iterator i = m_buf.begin() ; i != end ; ++i){
			m_maxlen = Max(m_maxlen,GetLayoutLen(*i));
		}
	}else m_maxlen = Max(m_maxlen,GetLayoutLen(*ite));
	return true;
}

const TCHAR *Memory::DeleteText(POS start,POS end,const TCHAR *sep){
	bool needanalyzelen = false;
	tmp_string = _T("");
	if(start > end)return NULL;
	if(start == end)return NULL;
	iterator ite = SearchLine(start.lineno);
	if(ite == m_buf.end())return NULL;

	if(start.column > ite->length())return NULL;

	if(start.lineno == end.lineno){
		tmp_string += ite->substr(start.column,end.column - start.column);
		int oldlen = ite->length();
		ite->erase(start.column,end.column - start.column);
		m_lastline = start.lineno;
		m_lastpos = ite;
		if(oldlen == m_maxlen){
			m_maxlen = 0;
			iterator end = m_buf.end();
			for(iterator i = m_buf.begin() ; i != end ; ++i){
				m_maxlen = Max(GetLayoutLen(*i),m_maxlen);
			}
		}
		if(m_buf.empty())m_buf.push_back(_T(""));
		return tmp_string.c_str();
	}

	if(ite->length() == m_maxlen)needanalyzelen = true;
	tmp_string = ite->substr(start.column) + _T("\n");
	ite->erase(start.column);
	++ite;
	if(ite == m_buf.end()){
		if(needanalyzelen){
			m_maxlen = 0;
			iterator end = m_buf.end();
			for(iterator i = m_buf.begin() ; i != end ; ++i){
				m_maxlen = Max(GetLayoutLen(*i),m_maxlen);
			}
		}
		return tmp_string.c_str();
	}
	tmp_string.reserve((end.lineno - start.lineno) * 50);
	for(int i = start.lineno + 1 ; i < end.lineno ; ++i){
		if(GetLayoutLen(*ite)  == m_maxlen)needanalyzelen = true;
		tmp_string += *ite + sep;
		ite = m_buf.erase(ite);
	}
	if(ite->length() == m_maxlen)needanalyzelen = true;
	tmp_string += ite->substr(0,end.column);
	ite->erase(0,end.column);
	iterator ite2 = ite;
	--ite;
	ite->append(*ite2);
	m_buf.erase(ite2);
	m_lastpos = ite;
	m_lastline = start.lineno;
	if(m_buf.empty())m_buf.push_back(_T(""));
	if(needanalyzelen){
		m_maxlen = 0;
		iterator end = m_buf.end();
		for(iterator i = m_buf.begin() ; i != end ; ++i){
			m_maxlen = Max(m_maxlen,GetLayoutLen(*i));
		}
	}
	return delstr.c_str();
}

Memory::iterator Memory::SearchLine(int line){
	int muki;
	iterator p;
	int i;
	int endl = m_buf.size() - 1;
	if(line < 0 || line > endl)return m_buf.end();
	if(line == 0)return m_buf.begin();
	if(m_lastpos != m_buf.begin()){
		if(line < m_lastline){
			if(2 * line > m_lastline){
				i = m_lastline;
				p = m_lastpos;
				muki = -1;
			}else{
				i = 0;
				p = m_buf.begin();
				muki = 1;
			}
		}else{
			if(2 * line > m_lastline + endl){
				i = endl;
				p = m_buf.end();
				--p;
				muki = -1;
			}else{
				i = m_lastline;
				p = m_lastpos;
				muki = 1;
			}
		}
	}else{
		if(2 * line > endl){
			i = endl;
			p = m_buf.end();
			--p;
			muki = -1;
		}else{
			i = 0;
			p = m_buf.begin();
			muki = 1;
		}
	}
	if(i == 0){++i;++p;}
	for(;(i - line) * muki < 0 && p != m_buf.begin() && p != m_buf.end() ; i += muki){
		if(muki > 0)++p;
		else --p;
	}
	if(p == m_buf.begin() || p == m_buf.end())return m_buf.end();
	return p;
}

void Memory::RemoveAll(){
	m_buf.clear();
	m_buf.push_back(_T(""));
	m_lastpos = m_buf.begin();
}

int Memory::GetLayoutLen(ablib::string &str){
	int tabs = 0;
	int p = 0,q = 0;
	const TCHAR *pp = str.c_str();
	while((p = str.find(_T("\t"),q)) != ablib::string::npos){
		int add = tabs + p + 1;
		for(; add % m_tabsize ; ++add);
		tabs = add - p;
		q = p + 1;
	}
	return tabs + str.length();
}

}//namespace edit
}//namespace window
}//namespace ablib