#include "edit.h"
#include "redoundo.h"

namespace ablib{
namespace window{
namespace edit{

void RedoUndo::SetMax(int max){
	if(max != m_max){
		Clear(m_head);
		m_hist = NULL;
		m_max = Max(max,1);
	}
}

void RedoUndo::AddUndoHist(Command *cmd){
	if(m_hist)Clear(m_hist->next);
	else Clear(m_head);
	Node *n = new Node(cmd,m_hist,NULL);
	if(m_head == NULL)m_head = n;
	if(m_hist){
		m_hist->next = n;
		m_hist = n;
	}else m_hist = n;
	if(m_max == m_size){
		Node *h = m_head;
		m_head = h->next;
		delete h;
		m_head->prev = NULL;
	}else ++m_size;
	++m_updated;
	DUMP(_T("AddHistory"));
	return;
}

void RedoUndo::Clear(Node *start){
	Node *p = start;
	if(start == NULL)return;
	else if(start == m_head)m_head = NULL;
	else if(start->prev)start->prev->next = NULL;
	Node *q;
	while(p){
		q = p->next;
		delete p;
		p = q;
		--m_size;
	}
}

bool RedoUndo::Undo(EditWnd *ed){
	if(m_hist == NULL)return false;//̐擪
	m_hist->SetCmd((*m_hist->cmd)(ed));
	m_hist = m_hist->prev;
	--m_updated;
	DUMP(_T("Undo"));
	return true;
}

bool RedoUndo::Redo(EditWnd *ed){
	if(m_hist == NULL && m_head == NULL)return false;//
	else if(m_hist == NULL)m_hist = m_head;//undoœɒBĂD
	else if(m_hist->next)m_hist = m_hist->next;//̐^
	else return false;//redooȂD
	m_hist->SetCmd((*m_hist->cmd)(ed));
	++m_updated;
	DUMP(_T("Redo"));
	return true;
}

}//namespace edit
}//namespace window
}//namespace ablib


//#define DUMP
#include <stdio.h>
namespace ablib{
namespace window{
namespace edit{
#ifdef DUMP
void RedoUndo::DUMP(const TCHAR *done){
	FILE *fp = ::_tfopen(_T("c:\\tmp\\test.txt"),_T("a"));
	Node *p = m_head;
	::_ftprintf(fp,_T("%s\n"),done);
	::_ftprintf(fp,_T("hist : %ld\n"),m_hist);
	while(p){
		::_ftprintf(fp,_T("%ld,%ld,%ld\n"),p,p->prev,p->next);
		p = p->next;
	}
	::_ftprintf(fp,_T("\n\n"));
	::fclose(fp);
}
#else
void RedoUndo::DUMP(const TCHAR*){}
#endif
}
}
}