#include "edit.h"
#include "search.h"
#include "edit_const.h"

namespace ablib{
namespace window{
namespace edit{

int Search::SearchDown(const ablib::string &str,const ablib::string &flag,POS &result){
	AnalyzeFlag(flag);
	if(!SetSearch(str,m_searchopt))return -1;
	if((m_searchopt & regular) && !m_reg.IsCompiled())return -1;
	return FindDown(result);
}

int Search::SearchUp(const ablib::string &str,const ablib::string &flag,POS &result){
	AnalyzeFlag(flag);
	if(!SetSearch(str,m_searchopt))return -1;
	if((m_searchopt & regular) && !m_reg.IsCompiled())return -1;
	return FindUp(result);
}

bool Search::SetSearch(const ablib::string &str,int flag){
	m_searchopt = flag;
	m_searchbuf = str;
	if(m_searchopt & regular){
		if(!m_reg.Compile(str))return false;
		if(m_searchopt & word)m_reg.SetOpt(_T("word"),TRUE);
		else m_reg.SetOpt(_T("word"),FALSE);
		if(m_searchopt & casesense)m_reg.SetOpt(_T("casesense"),TRUE);
		else m_reg.SetOpt(_T("casesense"),FALSE);
		if(m_searchopt & fuzzy)m_reg.SetOpt(_T("aimai"),TRUE);
		else m_reg.SetOpt(_T("aimai"),FALSE);
	}
	return true;
}

bool Search::SetSearch(const ablib::string &str,const ablib::string &flag){
	AnalyzeFlag(flag);
	return SetSearch(str,m_searchopt);
}

void Search::AnalyzeFlag(const ablib::string &flag){
	m_searchopt = 0;

	DEBUGSTRING(_T("window::Search : 󂯎IvV %s"),flag.c_str());

	if(flag.find(_T("word"))      != ablib::string::npos)m_searchopt |= word;
	if(flag.find(_T("replace"))   != ablib::string::npos)m_searchopt |= replace;
	if(flag.find(_T("ask"))       != ablib::string::npos)m_searchopt |= ask;
	if(flag.find(_T("regular"))   != ablib::string::npos)m_searchopt |= regular;
	if(flag.find(_T("fuzzy"))     != ablib::string::npos)m_searchopt |= fuzzy;
	if(flag.find(_T("inselect"))  != ablib::string::npos)m_searchopt |= inselect;
	if(m_searchopt & regular){
		if(flag.find(_T("nocasesense")) == ablib::string::npos)m_searchopt |= casesense;
	}else{
		if(flag.find(_T("casesense")) != ablib::string::npos
			&& flag.find(_T("nocasesense")) == ablib::string::npos)m_searchopt |= casesense;
	}
}

ablib::string Search::SearchOptionStr(){
	ablib::string rv;
	if(m_searchopt & word     )rv += _T("word,");
	if(m_searchopt & replace  )rv += _T("replace,");
	if(m_searchopt & ask      )rv += _T("ask,");
	if(m_searchopt & regular  )rv += _T("regular,");
	if(m_searchopt & fuzzy    )rv += _T("fuzzy,");
	if(m_searchopt & inselect )rv += _T("inselect,");
	if(m_searchopt & casesense)rv += _T("casesense,");
	else                       rv += _T("nocasesense,");
	return rv;
}

int Search::FindDown(POS &result){
	if(!(m_searchopt & casesense)){
		ablib::string str = m_searchbuf;
		m_searchbuf.tolower();
		int rv = FindDownHelper(result);
		m_searchbuf = str;
		return rv;
	}else return FindDownHelper(result);
}

int Search::FindUp(POS &result){
	if(!(m_searchopt & casesense)){
		ablib::string str = m_searchbuf;
		m_searchbuf.tolower();
		int rv = FindUpHelper(result);
		m_searchbuf = str;
		return rv;
	}else return FindUpHelper(result);
}

Search::FindLineData Search::FindLineDownReg(const ablib::string &str){
	FindLineData rv;
	if(m_reg.Match(str)){
		rv.length = m_reg.GetMatchLen();
		rv.pos = m_reg.GetMatchPos();
	}else rv.length = -1;
	return rv;
}

Search::FindLineData Search::FindLineUpReg(const ablib::string &str){
	int p = 0;
	FindLineData rv,fld;
	::ZeroMemory(&fld,sizeof(FindLineData));
	fld.length = -1;
	do{
		rv = fld;
		fld = FindLineDownReg(str.substr(p));
		p += fld.pos + 1;
	}while(fld.length != -1);
	return rv;
}

Search::FindLineData Search::FindLineDown(const ablib::string &str){
	FindLineData rv;
	int p;
	ablib::string target;
	target = str;
	if(!(m_searchopt & casesense))target.tolower();
	if(m_searchopt & word){
		int pos = 0;
		while(true){
			p = target.find(m_searchbuf,pos);
			if(p == ablib::string::npos){
				rv.length = -1;
				rv.pos = -1;
				return rv;
			}
			if(p == 0 || !IsEitango(target[p-1]) || !IsEitango(target[p + m_searchbuf.length()])){
				rv.length = m_searchbuf.length();
				rv.pos = p;
				m_findstr = target.substr(p,rv.length);
				return rv;
			}
			pos += p + 1;
		}
	}else{
		p = target.find(m_searchbuf);
		if(p != ablib::string::npos){
			rv.pos = p;
			rv.length = m_searchbuf.length();
			m_findstr = target.substr(p,rv.length);
		}else{
			rv.length = -1;
			rv.pos = -1;
		}
		return rv;
	}
}

Search::FindLineData Search::FindLineUp(const ablib::string &str){
	int p = 0;
	FindLineData rv,fld;
	::ZeroMemory(&fld,sizeof(FindLineData));
	fld.length = -1;
	do{
		rv = fld;
		fld = FindLineDown(str.substr(p));
		p += fld.pos + 1;
	}while(fld.length != -1);
	rv.pos = p - 1;
	return rv;
}

//悤Ȃ̂l[
int Search::FindDownHelper(POS &result){
	const TCHAR *str;
	ablib::string text;
	FindLineData fld;
	POS start,end;
	if(m_start == POS(-1,-1))start = POS(0,0);
	else start = m_start;
	if(m_end == POS(-1,-1)){
		end.lineno = m_buf->GetLineCount() - 1;
		str = m_buf->GetText(end.lineno);
		end.column = ::lstrlen(str);
	}else end = m_end;
	if(start > end){
		POS t = start;
		start = end;
		end = t;
	}
	if(m_searchopt & regular){
		if(start.lineno == end.lineno){
			str = m_buf->GetText(start.lineno);
			if(str == NULL)return -1;
			text = str + start.column;
			text.erase(end.column);
			fld = FindLineDownReg(text.c_str());
			if(fld.length != -1){
				result.lineno = start.lineno;
				result.column = start.column + fld.pos;
				return fld.length;
			}else return -1;
		}else{
			str = m_buf->GetText(start.lineno);
			if(str == NULL)return -1;
			text = str;
			text.erase(0,start.column);
			fld = FindLineDownReg(text.c_str());
			if(fld.length != -1){
				result.lineno = start.lineno;
				result.column = start.column + fld.pos;
				return fld.length;
			}
			for(int line = start.lineno + 1 ; line < end.lineno ; ++line){
				str = m_buf->GetText(line);
				if(str == NULL)return -1;
				fld = FindLineDownReg(str);
				if(fld.length != -1){
					result.lineno = line;
					result.column = fld.pos;
					return fld.length;
				}
			}
			str = m_buf->GetText(end.lineno);
			if(str == NULL)return -1;
			fld = FindLineDownReg(str);
			if(fld.length != -1){
				result.lineno = end.lineno;
				result.column = fld.pos;
				return fld.length;
			}else return -1;
		}
	}else{
		if(start.lineno == end.lineno){
			str = m_buf->GetText(start.lineno);
			if(str == NULL)return -1;
			text = str + start.column;
			text.erase(end.column);
			fld = FindLineDown(text.c_str());
			if(fld.length != -1){
				result.lineno = start.lineno;
				result.column = start.column + fld.pos;
				return fld.length;
			}else return -1;
		}else{
			str = m_buf->GetText(start.lineno);
			if(str == NULL)return -1;
			text = str;
			text.erase(0,start.column);
			fld = FindLineDown(text.c_str());
			if(fld.length != -1){
				result.lineno = start.lineno;
				result.column = start.column + fld.pos;
				return fld.length;
			}
			for(int line = start.lineno + 1 ; line < end.lineno ; ++line){
				str = m_buf->GetText(line);
				if(str == NULL)return -1;
				fld = FindLineDown(str);
				if(fld.length != -1){
					result.lineno = line;
					result.column = fld.pos;
					return fld.length;
				}
			}
			str = m_buf->GetText(end.lineno);
			if(str == NULL)return -1;
			fld = FindLineDown(str);
			if(fld.length != -1){
				result.lineno = end.lineno;
				result.column = fld.pos;
				return fld.length;
			}else return -1;
		}
	}
}

int Search::FindUpHelper(POS &result){
	const TCHAR *str;
	ablib::string text;
	FindLineData fld;
	POS start,end;
	if(m_end == POS(-1,-1))end = POS(0,0);
	else end = m_end;
	if(m_start == POS(-1,-1)){
		start.lineno = m_buf->GetLineCount() - 1;
		str = m_buf->GetText(start.lineno);
		start.column = ::lstrlen(str);
	}else start = m_start;
	if(start < end){
		POS t = end;
		end = start;
		start = t;
	}
	if(m_searchopt & regular){
		if(end.lineno == start.lineno){
			str = m_buf->GetText(end.lineno);
			if(str == NULL)return -1;
			text = str + end.column;
			text.erase(start.column);
			fld = FindLineUpReg(text.c_str());
			if(fld.length != -1){
				result.lineno = end.lineno;
				result.column = end.column + fld.pos;
				return fld.length;
			}else return -1;
		}else{
			str = m_buf->GetText(start.lineno);
			if(str == NULL)return -1;
			text = str;
			text.erase(start.column);
			fld = FindLineUpReg(text.c_str());
			if(fld.length != -1){
				result.lineno = start.lineno;
				result.column = fld.pos;
				return fld.length;
			}
			for(int line = start.lineno - 1 ; line > end.lineno ; --line){
				str = m_buf->GetText(line);
				if(str == NULL)return -1;
				fld = FindLineUpReg(str);
				if(fld.length != -1){
					result.lineno = line;
					result.column = fld.pos;
					return fld.length;
				}
			}
			str = m_buf->GetText(end.lineno);
			if(str == NULL)return -1;
			text = str;
			text.erase(0,end.column);
			fld = FindLineUpReg(text.c_str());
			if(fld.length != -1){
				result.lineno = end.lineno;
				result.column = end.column + fld.pos;
				return fld.length;
			}else return -1;
		}
	}else{
		if(end.lineno == start.lineno){
			str = m_buf->GetText(end.lineno);
			if(str == NULL)return -1;
			text = str + end.column;
			text.erase(start.column);
			fld = FindLineUp(text.c_str());
			if(fld.length != -1){
				result.lineno = end.lineno;
				result.column = end.column + fld.pos;
				return fld.length;
			}else return -1;
		}else{
			str = m_buf->GetText(start.lineno);
			if(str == NULL)return -1;
			text = str;
			text.erase(start.column);
			fld = FindLineUp(text.c_str());
			if(fld.length != -1){
				result.lineno = start.lineno;
				result.column = fld.pos;
				return fld.length;
			}
			for(int line = start.lineno - 1 ; line > end.lineno ; --line){
				str = m_buf->GetText(line);
				if(str == NULL)return -1;
				fld = FindLineUp(str);
				if(fld.length != -1){
					result.lineno = line;
					result.column = fld.pos;
					return fld.length;
				}
			}
			str = m_buf->GetText(end.lineno);
			if(str == NULL)return -1;
			text = str;
			text.erase(0,end.column);
			fld = FindLineUp(text.c_str());
			if(fld.length != -1){
				result.lineno = end.lineno;
				result.column = end.column + fld.pos;
				return fld.length;
			}else return -1;
		}
	}
}

ablib::string Search::GetReplaceStr(const ablib::string &getstr){
	if(m_searchopt & regular)return m_reg.GetMatchStr(getstr.c_str());
	else return getstr;
}

}//namespace edit
}//namespace window
}//namespace ablib