#include "ablib_system.h"
#include "ddesrv.h"
#include <map>

namespace ablib{
namespace system{


extern DWORD g_lasteerror;

std::map<HSZ,DdeSrv*> DdeSrv::m_topic2pt;
std::map<HCONV,DdeSrv*> DdeSrv::m_hconv2pt;

bool DdeSrv::Init(){
	if(m_servicestr == _T("") || m_topicstr == _T("")){SetLastError(ERROR_SUCCESS);return false;}
	if(::DdeInitialize(&m_ddeinst,DdeProc,APPCLASS_STANDARD,0) != DMLERR_NO_ERROR){SetLastError();return false;}
#ifdef UNICODE
	m_service = ::DdeCreateStringHandle(m_ddeinst,m_servicestr.c_str(),CP_WINUNICODE);
	m_topic = ::DdeCreateStringHandle(m_ddeinst,m_topicstr.c_str(),CP_WINUNICODE);
#else
	m_service = ::DdeCreateStringHandle(m_ddeinst,m_servicestr.c_str(),CP_WINANSI);
	m_topic = ::DdeCreateStringHandle(m_ddeinst,m_topicstr.c_str(),CP_WINANSI);
#endif
	m_topic2pt[m_topic] = this;
	if(!::DdeNameService(m_ddeinst,m_service,0L,DNS_REGISTER)){
		SetLastError();
		::DdeFreeStringHandle(m_ddeinst,m_service);
		::DdeFreeStringHandle(m_ddeinst,m_topic);
		::DdeUninitialize(m_ddeinst);
		m_ddeinst = 0;
		return false;
	}else return true;
}

bool DdeSrv::UnInit(){
	if(m_ddeinst != 0){
		m_topic2pt[m_topic] = NULL;
		::DdeNameService(m_ddeinst,m_service,0L,DNS_UNREGISTER);
		::DdeFreeStringHandle(m_ddeinst,m_service);
		::DdeFreeStringHandle(m_ddeinst,m_topic);
		::DdeUninitialize(m_ddeinst);
		m_ddeinst = 0;
	}
	return true;
}

HDDEDATA CALLBACK DdeSrv::DdeProc(UINT uType, UINT mFmt, HCONV hconv, HSZ hsz1, HSZ hsz2, HDDEDATA hdata, ULONG_PTR dwData1, ULONG_PTR dwData2) {
	TCHAR buf[512];
	DdeSrv *target;
	switch (uType) {
	case XTYP_CONNECT:
		target = m_topic2pt[hsz2];//Ƒv
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		TransDdeStr(target->m_ddeinst, hsz1, buf, sizeof(buf));
		if (target->m_servicestr != buf)return reinterpret_cast<HDDEDATA>(FALSE);
		return reinterpret_cast<HDDEDATA>(TRUE);

	case XTYP_CONNECT_CONFIRM:
		target = m_topic2pt[hsz2];//Ƒv
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		TransDdeStr(target->m_ddeinst, hsz1, buf, sizeof(buf));
		if (target->m_servicestr != buf)return reinterpret_cast<HDDEDATA>(FALSE);
		m_hconv2pt[hconv] = target;
		target->m_connect = true;
		return reinterpret_cast<HDDEDATA>(TRUE);

	case XTYP_DISCONNECT:
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		target->m_connect = false;
		target->m_advice = false;
		m_hconv2pt.erase(hconv);
		return reinterpret_cast<HDDEDATA>(TRUE);

	case XTYP_POKE:
	{
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		TransDdeStr(target->m_ddeinst, hsz2, buf, sizeof(buf));//R}h
		DDEPOKE ddepoke;
		ddepoke = target->m_ddepoke[buf];
		if (ddepoke) {
			DWORD size;
			TCHAR *arg = reinterpret_cast<TCHAR*>(::DdeAccessData(hdata, &size));
			HDDEDATA rv = ddepoke(arg) ? reinterpret_cast<HDDEDATA>(DDE_FACK) : reinterpret_cast<HDDEDATA>(FALSE);
			::DdeUnaccessData(hdata);
			::DdeFreeDataHandle(hdata);
			return rv;
		} else return reinterpret_cast<HDDEDATA>(FALSE);
	}
	case XTYP_EXECUTE:
	{
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		DWORD size;
		TCHAR *execname = reinterpret_cast<TCHAR*>(::DdeAccessData(hdata, &size));
		DDEEXECUTE ddeexecute;
		ddeexecute = target->m_ddeexec[execname];
		::DdeUnaccessData(hdata);
		::DdeFreeDataHandle(hdata);
		if (!ddeexecute)return reinterpret_cast<HDDEDATA>(FALSE);
		else if (ddeexecute())reinterpret_cast<HDDEDATA>(DDE_FACK);
		else return reinterpret_cast<HDDEDATA>(FALSE);
	}
	case XTYP_REQUEST:
	{
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		TransDdeStr(target->m_ddeinst, hsz2, buf, sizeof(buf));//item
		DDEREQUEST dderequest;
		dderequest = target->m_dderequest[buf];
		if (!dderequest)return reinterpret_cast<HDDEDATA>(FALSE);
		ablib::string rv = dderequest();
		return ::DdeCreateDataHandle(target->m_ddeinst,
			reinterpret_cast<LPBYTE>(const_cast<TCHAR*>(rv.c_str())),
			(DWORD)rv.length() + 1, 0, hsz2, CF_TEXT, 0);
	}
	case XTYP_ADVSTART:
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		if (target->m_advice)return reinterpret_cast<HDDEDATA>(FALSE);
		target->m_advice = true;
		return reinterpret_cast<HDDEDATA>(TRUE);

	case XTYP_ADVSTOP:
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		if (!target->m_advice)return reinterpret_cast<HDDEDATA>(FALSE);
		target->m_advice = false;
		return reinterpret_cast<HDDEDATA>(TRUE);

	case XTYP_ADVREQ:
		target = m_hconv2pt[hconv];
		if (!target)return reinterpret_cast<HDDEDATA>(FALSE);
		if (!target->m_advice)return reinterpret_cast<HDDEDATA>(FALSE);
		return ::DdeCreateDataHandle(target->m_ddeinst,
			reinterpret_cast<LPBYTE>(const_cast<TCHAR*>(target->m_advicestr.c_str())),
			(DWORD)target->m_advicestr.length() + 1, 0, hsz2, CF_TEXT, 0);

	default:
		return reinterpret_cast<HDDEDATA>(FALSE);
	}
}

bool DdeSrv::PostAdvice(const TCHAR *itemstr,const TCHAR *data){
	m_advicestr = data;
#ifdef UNICODE
	HSZ item = ::DdeCreateStringHandle(m_ddeinst,itemstr,CP_WINUNICODE);
#else
	HSZ item = ::DdeCreateStringHandle(m_ddeinst,itemstr,CP_WINANSI);
#endif
	BOOL rv = ::DdePostAdvise(m_ddeinst,m_topic,item);
	SetLastError();
	::DdeFreeStringHandle(m_ddeinst,item);
	return rv ? true : false;
}


}//namespace system
}//namespace ablib