#include "ablib_system.h"
#include "registory.h"
#include <vector>

namespace ablib{
namespace system{


bool Registory::Open(HKEY key,const TCHAR *path){
	Close();
	if(::RegOpenKeyEx(key,path,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,&m_handle) == ERROR_SUCCESS){
		m_root = key;
		m_path = path;
		return true;
	}else{
		SetLastError();
		return false;
	}
}

bool Registory::Create(HKEY key,const TCHAR *path){
	Close();
	if(::RegCreateKeyEx(key,path,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&m_handle,NULL) == ERROR_SUCCESS){
		m_root = key;
		m_path = path;
		return true;
	}else{
		SetLastError();
		return false;
	}
}

ablib::string Registory::ReadStr(const TCHAR *entry){
	DWORD type,size;
	if(::RegQueryValueEx(m_handle,entry,NULL,&type,NULL,&size) != ERROR_SUCCESS){SetLastError();return _T("");}
	if(type != REG_SZ && type != REG_EXPAND_SZ){SetLastError(ERROR_SUCCESS);return _T("");}
	TCHAR *s = new TCHAR[size + 1];++size;
	if(::RegQueryValueEx(m_handle,entry,NULL,&type,reinterpret_cast<BYTE*>(s),&size) != ERROR_SUCCESS){
		SetLastError();
		delete[] s;
		return _T("");
	}
	ablib::string rv = s;
	delete[] s;
	return rv;
}

DWORD Registory::ReadDWORD(const TCHAR *entry){
	DWORD rv,size = sizeof(DWORD),type;
	if(::RegQueryValueEx(m_handle,entry,NULL,&type,reinterpret_cast<BYTE*>(&rv),&size) != ERROR_SUCCESS){SetLastError();return 0;}
	if(type != REG_DWORD && type != REG_DWORD_LITTLE_ENDIAN){SetLastError(ERROR_SUCCESS);return 0;}
	else return rv;
}

bool Registory::WriteStr(const TCHAR *entry,const ablib::string &str){
	if(::RegSetValueEx(m_handle,entry,NULL,REG_SZ,reinterpret_cast<BYTE*>(const_cast<TCHAR*>(str.c_str())),(DWORD)(str.length())*sizeof(TCHAR)) == ERROR_SUCCESS)return true;
	else{SetLastError();return false;}
}

bool Registory::WriteStrExpand(const TCHAR *entry,const ablib::string &str){
	if(::RegSetValueEx(m_handle,entry,NULL,REG_EXPAND_SZ,reinterpret_cast<BYTE*>(const_cast<TCHAR*>(str.c_str())),(DWORD)(str.length())*sizeof(TCHAR)) == ERROR_SUCCESS)return true;
	else{SetLastError();return false;}
}

bool Registory::WriteDWORD(const TCHAR *entry,DWORD val){
	if(::RegSetValueEx(m_handle,entry,NULL,REG_DWORD,reinterpret_cast<BYTE*>(&val),sizeof(DWORD)) == ERROR_SUCCESS)return true;
	else{SetLastError();return false;}
}

bool Registory::Delete(){
	Close();
	return DeleteKey(m_root,m_path);
}

bool Registory::Delete(const TCHAR *entry){
	if(::RegDeleteValue(m_handle,entry) == ERROR_SUCCESS)return true;
	else{SetLastError();return false;}
}

bool Registory::DeleteKey(HKEY key,const ablib::string &path){
	HKEY handle;
	if(::RegOpenKeyEx(key,path.c_str(),0,KEY_ALL_ACCESS,&handle) != ERROR_SUCCESS){SetLastError();return false;}
	DWORD size = 1023;
	TCHAR s[1024];

	std::vector<ablib::string> keys;

	for(int i = 0 ;::RegEnumKeyEx(handle,i,s,&size,0,NULL,NULL,NULL) != ERROR_NO_MORE_ITEMS ; ++i){
		keys.push_back(path + _T("\\") + s);
		size = 1023;
	}
	for(std::vector<ablib::string>::size_type i = 0 ; i < keys.size() ; ++i){
		DeleteKey(key,keys[i]);
	}

	std::vector<ablib::string> values;
	size = 1023;
	for(int i = 0 ; ::RegEnumValue(handle,i,s,&size,0,NULL,NULL,NULL) != ERROR_NO_MORE_ITEMS ; ++i){
		values.push_back(s);
	}
	for(std::vector<ablib::string>::size_type i = 0 ; i < values.size() ; ++i)::RegDeleteValue(handle,values[i].c_str());
	::RegCloseKey(handle);
	if(::RegDeleteKey(key,path.c_str()) == ERROR_SUCCESS)return true;
	else{SetLastError();return false;}
}

bool Registory::EnumKeys(std::vector<ablib::string> &rv){
	TCHAR str[4096];
	DWORD size = 4096;
	for(int i = 0 ; ; ++i){
		::lstrcpy(str,_T(""));
		size = 4096;
		if(::RegEnumKeyEx(m_handle,i,str,&size,0,NULL,NULL,NULL) == ERROR_NO_MORE_ITEMS)break;
		rv.push_back(str);
	}
	return true;
}

}//namespace system
}//namespace ablib

