#include "ablib_system.h"
#include "time.h"


namespace{
	TCHAR tmp[8];
	inline const TCHAR *Add0(DWORD d){
		if(d < 10){tmp[0] = '0';::wsprintf(tmp + 1,_T("%d"),d);}
		else ::wsprintf(tmp,_T("%d"),d);
		return tmp;
	}
}

namespace ablib{
namespace system{

Time::Time(int year,int month,int day,int hour,int minute,int second,int millisecond){
	m_st.wYear = year;m_st.wMonth = month;m_st.wDay = day;
	m_st.wHour = hour;m_st.wMinute = minute;m_st.wSecond = second;
	m_st.wMilliseconds = millisecond;
	m_st.wDayOfWeek = GetYoubi(year,month,day);
	m_valid = CheckValid();
}

ablib::string Time::ToStr(){
	if(m_valid){
		ablib::string rv;
		rv.reserve(128);
		TCHAR buf[8];
		::wsprintf(buf,_T("%d"),m_st.wYear);
		rv += buf;
		rv += _T("-");
		rv += Add0(m_st.wMonth);
		rv += _T("-");
		rv += Add0(m_st.wDay);
		rv += _T("T");
		rv += Add0(m_st.wHour);
		rv += _T(":");
		rv += Add0(m_st.wMinute);
		rv += _T(":");
		rv += Add0(m_st.wSecond);
//		rv += _T("+09:00");
		return rv;
	}else return ablib::string(_T(""));
}

const TCHAR *weakstr[] = {_T("Sun"),_T("Mon"),_T("Tue"),_T("Wed"),_T("Thu"),_T("Fri"),_T("Sat")};
const TCHAR *monthstr[] = 
	{_T("Jan"),_T("Feb"),_T("Mar"),_T("Apr"),_T("May"),_T("Jun"),
	 _T("Jul"),_T("Aug"),_T("Sep"),_T("Oct"),_T("Nov"),_T("Dec")};

ablib::string Time::RFC1123String(){
	if(!IsValid())return _T("");
	ablib::string rv;
	rv.reserve(64);
	TCHAR tmp[64];
	rv += weakstr[m_st.wDayOfWeek];
	rv += _T(", ");
	rv += Add0(m_st.wDay);
	rv += _T(" ");
	rv += monthstr[m_st.wMonth - 1];
	rv += _T(" ");
	::wsprintf(tmp,_T("%d "),m_st.wYear);
	rv += tmp;
	rv += Add0(m_st.wHour);
	rv += _T(":");
	rv += Add0(m_st.wMinute);
	rv += _T(":");
	rv += Add0(m_st.wSecond);
	rv += _T(" GMT");
	return rv;
}

bool Time::operator<(const Time &t) const{
	if(!t.IsValid())return false;
	else if(t.IsValid() && !m_valid)return true;
	else{
		FILETIME ftme,ftobj;
		::SystemTimeToFileTime(&m_st,&ftme);
		::SystemTimeToFileTime(&(t.m_st),&ftobj);
		return ::CompareFileTime(&ftme,&ftobj) < 0;
	}
}

bool Time::operator==(const Time &t) const{
	if(!t.IsValid() && !m_valid)return true;
	else if(!t.IsValid() && m_valid)return false;
	else if(t.IsValid() && !m_valid)return false;
	else{
		FILETIME ftme,ftobj;
		::SystemTimeToFileTime(&m_st,&ftme);
		::SystemTimeToFileTime(&(t.m_st),&ftobj);
		return ::CompareFileTime(&ftme,&ftobj) == 0;
	}
}

namespace{
	// lhs + rhsvZCkuriȏȂ炻̕ueketaɌJオ
	inline void CalcKuriagari(WORD &lhs,const WORD &rhs,WORD &ueketa,WORD kuri){
		lhs += rhs;
		ueketa += lhs / kuri;
		lhs -= kuri * (lhs / kuri);
		return;
	}

	// {Iɂ͌JオvZCkuri̔{͈̎ȂJオ
	inline void CalcKuriagari2(WORD &lhs,const WORD &rhs,WORD &ueketa,WORD kuri){
		lhs += rhs;
		ueketa += lhs / kuri;
		lhs -= kuri * (lhs / kuri);
		if(lhs == 0){
			--ueketa;
			lhs = kuri;
		}
		return;
	}

}

Time &Time::operator+=(const Time &obj){
	if(!IsValid())return *this;
	CalcKuriagari(m_st.wMilliseconds,obj.m_st.wMilliseconds,m_st.wSecond,1000);
	CalcKuriagari(m_st.wSecond,obj.m_st.wSecond,m_st.wMinute,60);
	CalcKuriagari(m_st.wMinute,obj.m_st.wMinute,m_st.wHour,60);
	CalcKuriagari(m_st.wHour,obj.m_st.wHour,m_st.wDay,24);
	CalcKuriagari2(m_st.wDay,obj.m_st.wDay,m_st.wMonth,32);
	CalcKuriagari2(m_st.wMonth,obj.m_st.wMonth,m_st.wYear,12);
	m_st.wYear += obj.m_st.wYear;
	if((m_st.wMonth == 4 || m_st.wMonth == 6 || m_st.wMonth == 9 || m_st.wMonth == 11) && m_st.wDay == 31){
		m_st.wDay = 1;
		++m_st.wMonth;
	}else if(m_st.wMonth == 2){
		if(IsUruu(m_st.wYear)){
			if(m_st.wDay > 29){
				m_st.wDay -= 29;
				m_st.wMonth = 3;
			}
		}else{
			if(m_st.wDay > 28){
				m_st.wDay -= 28;
				m_st.wMonth = 3;
			}
		}
	}
	return *this;
}

bool Time::CheckValid(){
	if(
		m_st.wYear >= 1601 &&
		(0 <= m_st.wMonth && m_st.wMonth <= 12) &&
		(0 <= m_st.wDayOfWeek && m_st.wDayOfWeek <= 7) &&
		(0 <= m_st.wDay && m_st.wDay <= 31) &&
		(0 <= m_st.wHour && m_st.wHour <= 23) &&
		(0 <= m_st.wMinute && m_st.wMinute <= 59) &&
		(0 <= m_st.wSecond && m_st.wSecond <= 59) &&
		(0 <= m_st.wMilliseconds && m_st.wMilliseconds <= 1000)
	){
		return true;
	}else{
		SetValid(false);
		return false;
	}
}

bool Time::IsUruu(int year){
	if(year % 4 != 0)return false;
	if(year % 100 != 0)return true;
	if(year % 400 != 0)return false;
	return true;
}

WORD Time::GetYoubi(int year,int month,int day){
	if(month == 1 || month == 2){
		--year;
		month += 12;
	}
	int h = year / 100;
	int y = year - 100 * h;
	int w = y;
	w += y/4;
	w += h/4;
	w += (13 * (month + 1) / 5);
	w += day - 2 * h - 1;
	w = w % 7;
	if(w < 0)w += 7;
	return w;
}

}//namespace system
}//namespace ablib

