#include "ablib_window.h"
#include "Dialog.h"
#include <boost/lexical_cast.hpp>

#pragma comment(lib, "comctl32.lib")

namespace ablib{
namespace window{

bool Dialog::SetFont(FontData f,bool redraw){
	HFONT font = ::CreateFont((int)f.height, (int)f.avewidth,0,0, (int)f.weight,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,FIXED_PITCH |FF_MODERN ,f.fontname.c_str());
	if(font == NULL)return false;
	::SendMessage(m_hwnd,WM_SETFONT,reinterpret_cast<WPARAM>(font),redraw ? TRUE : FALSE);
	if(m_font)::DeleteObject(m_font);
	m_font = font;
	GetFontData();
	return true;
}

FontData Dialog::GetFont(){
	if(m_fontdata.fontname == _T(""))GetFontData();
	return m_fontdata;
}

void Dialog::GetFontData(){
	HFONT hfont = reinterpret_cast<HFONT>(::SendMessage(m_hwnd,WM_GETFONT,0,0));
	LOGFONT lf;
	::GetObject(hfont,sizeof(LOGFONT),&lf);
	m_fontdata = FontData(lf.lfFaceName,lf.lfHeight,lf.lfWeight,lf.lfWidth);

/*
	HDC hdc = ::GetDC(m_hwnd);
	if(hdc){
		TEXTMETRIC tm;
		::GetTextMetrics(hdc,&tm);
		::ReleaseDC(m_hwnd,hdc);
		m_fontdata.avewidth = tm.tmAveCharWidth;
		m_fontdata.height = tm.tmHeight;
		m_fontdata.weight = tm.tmWeight;
	}
*/
}

bool ToolBar::Create(Wnd *parent,int id,const std::vector<WORD> &ids,Point size,HINSTANCE appint,int style){
	int num;
	TBBUTTON *btns = CreateTBButton(ids,&num);

	bool rv = ::CreateToolbarEx(parent ? parent->GetHWND() : NULL,style,id,
	num,appint,id,btns,(int)ids.size(),0,0, (int)size.x, (int)size.y,sizeof(TBBUTTON)) != NULL;

	delete[] btns;
	return rv;
}

bool ToolBar::CreateSysToolBar(Wnd *parent,int id,const std::vector<SysButton> &ids,Point size,int style){
	int num;
	TBBUTTON *btns = CreateTBButton(ids,&num);

	bool rv = ::CreateToolbarEx(parent ? parent->GetHWND() : NULL,style,id,
	num,reinterpret_cast<HINSTANCE>(HINST_COMMCTRL),IDB_STD_SMALL_COLOR,btns,(int)ids.size(),0,0, (int)size.x, (int)size.y,sizeof(TBBUTTON)) != NULL;

	delete[] btns;
	return rv;
}

void ToolBar::Add(const std::vector<WORD> &ids,int id,HINSTANCE appint){
	TBADDBITMAP tb;
	tb.hInst = appint;tb.nID = id;
	int num;
	TBBUTTON *btns = CreateTBButton(ids,&num);
	intptr_t index = ::SendMessage(m_hwnd,TB_ADDBITMAP , num , reinterpret_cast<LPARAM>(&tb));
	for(std::vector<WORD>::size_type i = 0 ; i < ids.size() ; ++i)btns[(int)i].iBitmap += (int)index;
	::SendMessage(m_hwnd,TB_ADDBUTTONS,ids.size(),reinterpret_cast<LPARAM>(btns));

	delete[] btns;
	return;
}


void ToolBar::SetToolTip(const std::vector<ToolTip> &tips,HINSTANCE appint){
	HWND parent = ::GetParent(m_hwnd);
	HWND tooltip = ::CreateWindowEx(0,TOOLTIPS_CLASS,NULL,TTS_ALWAYSTIP,
		CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,
		parent,NULL,appint,NULL);

	TOOLINFO ti;
	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = m_hwnd;
	ti.hinst = appint;
	ti.uFlags = TTF_SUBCLASS;
	for(std::vector<ToolTip>::size_type i = 0 ; i < tips.size() ; ++i){
		::SendMessage(m_hwnd,TB_GETITEMRECT,i,reinterpret_cast<LPARAM>(&ti.rect));
		ti.uId = tips[i].buttonid;
		ti.lpszText = MAKEINTRESOURCE(tips[i].stringid);
		::SendMessage(tooltip,TTM_ADDTOOL,0,reinterpret_cast<LPARAM>(&ti));
	}
	::SendMessage(m_hwnd,TB_SETTOOLTIPS,reinterpret_cast<WPARAM>(tooltip),0);
	return;
}

TBBUTTON *ToolBar::CreateTBButton(const std::vector<WORD> &ids,int *num){
	TBBUTTON *btns = new TBBUTTON[ids.size()];
	*num = 0;
	for(std::vector<WORD>::size_type i = 0 ; i < ids.size() ; ++i){
		::ZeroMemory(btns + i,sizeof(TBBUTTON));
		if(ids[i] == 0){
			btns[(int)i].iBitmap = (int)i;btns[(int)i].idCommand = 0;btns[(int)i].fsState = TBSTATE_ENABLED;
			btns[(int)i].fsStyle = TBSTYLE_SEP;btns[(int)i].dwData = btns[(int)i].iString = 0;
		}else{
			++(*num);
			btns[(int)i].iBitmap = (int)i;btns[(int)i].idCommand = ids[i];btns[(int)i].fsState = TBSTATE_ENABLED;
			btns[(int)i].fsStyle = TBSTYLE_BUTTON;btns[i].dwData = btns[(int)i].iString = 0;
		}
	}
	return btns;
}

TBBUTTON *ToolBar::CreateTBButton(const std::vector<SysButton> &ids,int *num){
	TBBUTTON *btns = new TBBUTTON[ids.size()];
	*num = 0;
	for(std::vector<WORD>::size_type i = 0 ; i < ids.size() ; ++i){
		::ZeroMemory(btns + i,sizeof(TBBUTTON));
		if(ids[i].id == 0){
			btns[(int)i].iBitmap = (int)i;btns[(int)i].idCommand = 0;btns[(int)i].fsState = TBSTATE_ENABLED;
			btns[(int)i].fsStyle = TBSTYLE_SEP;btns[(int)i].dwData = btns[(int)i].iString = 0;
		}else{
			++(*num);
			btns[(int)i].iBitmap = ids[i].button;btns[(int)i].idCommand = ids[i].id;btns[(int)i].fsState = TBSTATE_ENABLED;
			btns[(int)i].fsStyle = TBSTYLE_BUTTON;btns[(int)i].dwData = btns[(int)i].iString = 0;
		}
	}
	return btns;
}


bool ReBar::Create(Wnd *parent,int id,HINSTANCE appint,int style){
	HWND hwnd;
	if((hwnd = ::CreateWindowEx(WS_EX_TOOLWINDOW,REBARCLASSNAME,NULL,style,0,0,100,0,parent ? parent->GetHWND() : NULL,reinterpret_cast<HMENU>((intptr_t)id),appint,NULL)) == NULL)return false;
	REBARINFO rbi;
	::ZeroMemory(&rbi,sizeof(REBARINFO));
	rbi.cbSize = sizeof(REBARINFO);
	return ::SendMessage(hwnd,RB_SETBARINFO,0,reinterpret_cast<LPARAM>(&rbi)) != 0;
}

bool ReBar::AddDlg(Dialog *dlg,const TCHAR *txt,Point size,int style){
	REBARBANDINFO rbbi;
	::ZeroMemory(&rbbi,sizeof(REBARBANDINFO));
	rbbi.cbSize = sizeof(REBARBANDINFO);
	rbbi.fMask = RBBIM_CHILD | RBBIM_TEXT | RBBIM_STYLE | RBBIM_CHILDSIZE | RBBIM_SIZE;
	rbbi.lpText = const_cast<TCHAR*>(txt);
	rbbi.cch = ::lstrlen(txt);
	rbbi.fStyle = style;
	rbbi.hwndChild = dlg->GetHWND();
	rbbi.cyMinChild = (UINT)size.y;rbbi.cyMaxChild = 100;
	rbbi.cxMinChild = 25;
	rbbi.cx = (UINT)size.x;rbbi.cyChild = (UINT)size.y;
	return SendMessage(m_hwnd,RB_INSERTBAND,-1,reinterpret_cast<LPARAM>(&rbbi)) != 0;
}

void List::SetHorizontalExtent(){
	size_t max = 0;
	intptr_t num = GetCount();
	ablib::string maxstr;
	ablib::string s;
	for(intptr_t  i = 0 ; i < num ; ++i){
		s = GetText(i);
		if(s.length() > max){
			max = s.length();
			maxstr = s;
		}
	}
	HDC hdc = ::GetDC(m_hwnd);
	int save = ::SaveDC(hdc);
	HFONT hfont = (HFONT)SendMessage(m_hwnd,WM_GETFONT,0,0);
	if(hfont)::SelectObject(hdc,hfont);
	SIZE size;
	::GetTextExtentPoint32(hdc,maxstr.c_str(),(int)maxstr.length(),&size);
	SetHorizontalExtent(size.cx + ::GetSystemMetrics(SM_CXVSCROLL));
	::RestoreDC(hdc,save);
	::ReleaseDC(m_hwnd,hdc);
	return;
}

HTREEITEM TreeView::Insert(const Item &item,HTREEITEM parent,HTREEITEM after){
	TVINSERTSTRUCT ins;
	::ZeroMemory(&ins,sizeof(TVINSERTSTRUCT));
	ins.hInsertAfter = after;
	ins.hParent = parent;
	if(item.str != _T("")){
		ins.item.mask |= TVIF_TEXT;
		ins.item.pszText = const_cast<TCHAR*>(item.str.c_str());
	}
	if(item.imageindex != -1){
		ins.item.iImage = ins.item.iSelectedImage = item.imageindex;
		ins.item.mask |= TVIF_IMAGE | TVIF_SELECTEDIMAGE;
	}
	return TreeView_InsertItem(m_hwnd,&ins);
}


}//namespace window
}//namespace ablib

