#ifndef __ABLIB_WINDOW_GRAPHIC_H__
#define __ABLIB_WINDOW_GRAPHIC_H__

#include "window.h"
#include <ablib/lexical_cast.h>

namespace ablib{
namespace window{

class Graphic{
public:
	explicit Graphic(Window *wnd);
	~Graphic();

	/// `ׂʒu̎擾
	Rect GetPaintRect();
	/// eLXgóD
	void WriteText(Point p,const TCHAR *text){WriteText(p,text,::lstrlen(text));}
	void WriteText(Point p,const ablib::string &text){WriteText(p,text.c_str(),text.length());}
	void WriteText(Point p,const TCHAR *text, intptr_t len);
	void WriteText(Point p,const ablib::string &text, intptr_t len){WriteText(p,text.c_str(),len);}
	/// D
	void DrawLine(Point p1,Point p2);
	/// lphԂ
	void FillRect(Rect rc);

	void SetUpdate();

private:
	Window *m_wnd;

	PAINTSTRUCT m_ps;
	HDC m_hdc;
	HGDIOBJ m_oldobject;
	enum objecttype{obj_none,obj_font,obj_line};
	objecttype m_objtype;
	
	HBRUSH m_fillrect;
	
	void SetObject(objecttype obj);

	NOCOPY(Graphic);
};

inline Graphic::Graphic(Window *wnd) : m_wnd(wnd),m_oldobject(NULL),m_objtype(obj_none){
	m_hdc = ::BeginPaint(wnd->GetHWND(),&m_ps);
	wnd->m_hdc_from_graphic = m_hdc;
	m_fillrect = ::CreateSolidBrush(RGB(m_wnd->GetFillColor().red,m_wnd->GetFillColor().green,m_wnd->GetFillColor().blue));
}

inline Graphic::~Graphic(){
	::DeleteObject(::SelectObject(m_hdc,m_oldobject));
	::DeleteObject(m_fillrect);
	::EndPaint(m_wnd->GetHWND(),&m_ps);
	m_wnd->m_hdc_from_graphic = NULL;
}

inline void Graphic::WriteText(Point p,const TCHAR *text,intptr_t len){
	SetObject(obj_font);
	::TextOut(m_hdc,(int)p.x,(int)p.y,text,(int)len);
}

inline void Graphic::DrawLine(Point p1,Point p2){
	SetObject(obj_line);
	::MoveToEx(m_hdc,(int)p1.x,(int)p1.y,NULL);
	::LineTo(m_hdc, (int)p2.x, (int)p2.y);
}

inline void Graphic::FillRect(Rect rc){
	RECT rect;
	rect.left = (int)rc.left;rect.right = (int)rc.right;
	rect.top = (int)rc.top;rect.bottom = (int)rc.bottom;
	::FillRect(m_hdc,&rect,m_fillrect);
}

inline void Graphic::SetObject(objecttype obj){
	if(m_objtype == obj)return;
	m_objtype = obj;
	SetUpdate();
}

inline void Graphic::SetUpdate(){
	if(m_objtype != obj_none){
		if(m_oldobject)::DeleteObject(::SelectObject(m_hdc,m_oldobject));
		HGDIOBJ newobj;
		switch(m_objtype){
		case obj_font:
		{
			ablib::window::FontData fd = m_wnd->GetFontData();
			newobj = static_cast<HGDIOBJ>(::CreateFont((int)fd.height, (int)fd.avewidth,0,0, (int)fd.weight,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,FIXED_PITCH |FF_MODERN ,fd.fontname.c_str()));
			m_oldobject = ::SelectObject(m_hdc,newobj);
			::GetTextMetrics(m_hdc,&(m_wnd->m_textmetric));
			m_wnd->m_fontdata = FontData(m_wnd->m_fontdata.fontname,m_wnd->m_textmetric.tmHeight,m_wnd->m_textmetric.tmWeight,m_wnd->m_textmetric.tmAveCharWidth);
//			::MessageBox(NULL,(ablib::string(_T("height = ")) + ablib::lexical_cast<ablib::string>(m_wnd->m_fontdata.height) + _T("\r\nwidth = ") + ablib::lexical_cast<ablib::string>(m_wnd->m_fontdata.avewidth)).c_str(),NULL,0);
			break;
		}
		case obj_line:
			newobj = static_cast<HGDIOBJ>(::CreatePen(m_wnd->GetLineType(),(int)m_wnd->GetLineWidth(),RGB(m_wnd->GetLineColor().red,m_wnd->GetLineColor().green,m_wnd->GetLineColor().blue)));
			m_oldobject = ::SelectObject(m_hdc,newobj);
			break;
		default:
			ASSERT(false);
		}
	}
	if(m_fillrect){
		::DeleteObject(m_fillrect);
		m_fillrect = ::CreateSolidBrush(RGB(m_wnd->GetFillColor().red,m_wnd->GetFillColor().green,m_wnd->GetFillColor().blue));
	}
	::SetTextColor(m_hdc,RGB(m_wnd->GetTextColor().red,m_wnd->GetTextColor().green,m_wnd->GetTextColor().blue));
	::SetBkColor(m_hdc,RGB(m_wnd->GetBackgroundColor().red,m_wnd->GetBackgroundColor().green,m_wnd->GetBackgroundColor().blue));
}

inline Rect Graphic::GetPaintRect(){
	return Rect(
		m_ps.rcPaint.left,
		m_ps.rcPaint.top,
		m_ps.rcPaint.right,
		m_ps.rcPaint.bottom
	);
}


}//namespace window
}//namespace ablib

#endif //__ABLIB_WINDOW_GRAPHIC_H__

