#include "ablib_window.h"
#include "PropSheet.h"

namespace ablib{
namespace window{

std::map<DWORD,PropSheet_Base*> PropSheet_Base::m_thread2pt;

int CALLBACK PropSheet_Base::WndProc(HWND hwndDlg,UINT uMsg,LPARAM lParam){
	if(uMsg == PSCB_INITIALIZED){
		DWORD thread = ::GetCurrentThreadId();
		PropSheet_Base *target = m_thread2pt[thread];
		ASSERT(target != NULL);
		target->m_sheet = new Sheet(hwndDlg);
		m_thread2pt.erase(thread);
	}
	return 0;
}

INT_PTR BasePropSheet::SheetWndProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam){
	SheetWnd *target = static_cast<SheetWnd*>(::GetProp(hwndDlg,_T("ABENORIPROPSHEETPROP")));
	if(!target){
		if(uMsg == WM_INITDIALOG)target = reinterpret_cast<SheetWnd*>(reinterpret_cast<PROPSHEETPAGE*>(lParam)->lParam);
		if(target)Atach(hwndDlg,target);
	}
	if(target){
		Dialog me(hwndDlg);
		switch(uMsg){
		case WM_INITDIALOG:	return target->OnInitDialog(&me) ? TRUE : FALSE;
		case WM_CLOSE:		return target->OnClose(&me) ? TRUE : FALSE;
		case WM_DESTROY:
			if(target->OnDestroy(&me)){
				Detach(hwndDlg);
				return TRUE;
			}else return FALSE;
		case WM_COMMAND:
			return target->OnCommand(&me,static_cast<int>(LOWORD(wParam)),static_cast<int>(HIWORD(wParam))) ? TRUE : FALSE;
		case WM_NOTIFY:
		{
			LPPSHNOTIFY lppshN = reinterpret_cast<LPPSHNOTIFY>(lParam);
			switch(lppshN->hdr.code){
			case PSN_SETACTIVE:	return target->OnSetActive(&me) ? TRUE : FALSE;
			case PSN_APPLY:		return target->OnApply(&me) ? TRUE : FALSE;
			}
			break;
		}
		}
		return target->OnOtherMessage(&me,uMsg,wParam,lParam) ? TRUE : FALSE;
	}
	return FALSE;
}

INT_PTR BaseWizard::SheetWndProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam){
	SheetWnd *target = static_cast<SheetWnd*>(::GetProp(hwndDlg,_T("ABENORIPROPSHEETPROP")));
	if(!target){
		if(uMsg == WM_INITDIALOG){
			target = reinterpret_cast<SheetWnd*>(reinterpret_cast<PROPSHEETPAGE*>(lParam)->lParam);
			target->m_window = new Dialog(hwndDlg);
		}
		if(target)Atach(hwndDlg,target);
	}
	if(target){
		Dialog *me = target->m_window;
		if(!me)return FALSE;
		switch(uMsg){
		case WM_INITDIALOG:	return target->OnInitDialog(me) ? TRUE : FALSE;
		case WM_CLOSE:		return target->OnClose(me) ? TRUE : FALSE;
		case WM_DESTROY:
			if(target->OnDestroy(me)){
				Detach(hwndDlg);
				delete me;
				target->m_window = NULL;
				return TRUE;
			}else return FALSE;
		case WM_COMMAND:
			return target->OnCommand(me,static_cast<int>(LOWORD(wParam)),static_cast<int>(HIWORD(wParam))) ? TRUE : FALSE;
		case WM_NOTIFY:
		{
			LPPSHNOTIFY lppshN = reinterpret_cast<LPPSHNOTIFY>(lParam);
			switch(lppshN->hdr.code){
			case PSN_SETACTIVE:
			{
//				if(target->m_finish)PropSheet_CancelToClose(hwndDlg);
				PropSheet_SetWizButtons(::GetParent(hwndDlg),
					((target->m_back ? PSWIZB_BACK : 0) |
					(target->m_finish ? PSWIZB_FINISH : (target->m_next ? PSWIZB_NEXT : 0)))
					);
				LPCTSTR rv = target->OnSetActive(me);
				if(rv == 0)return 0;
				::SetWindowLongPtr(hwndDlg,DWLP_MSGRESULT,reinterpret_cast<LONG_PTR>(rv));
				return TRUE;
			}
			case PSN_WIZNEXT:
			{
				target->GetParentPropSheet()->m_previousid = target->m_id;
				LPCTSTR rv = target->OnWizNext(me);
				if(rv == 0)return 0;
				::SetWindowLongPtr(hwndDlg,DWLP_MSGRESULT,reinterpret_cast<LONG_PTR>(rv));
				return TRUE;
			}
			case PSN_WIZBACK:
			{
				target->GetParentPropSheet()->m_previousid = target->m_id;
				LPCTSTR rv = target->OnWizBack(me);
				if(rv == 0)return 0;
				::SetWindowLongPtr(hwndDlg,DWLP_MSGRESULT,reinterpret_cast<LONG_PTR>(rv));
				return TRUE;
			}
			case PSN_WIZFINISH:{
				target->GetParentPropSheet()->m_previousid = target->m_id;
				return target->OnWizFinish(me) ? TRUE : FALSE;
			}
			break;
			}
		}
		default:
			break;
		}
		return target->OnOtherMessage(me,uMsg,wParam,lParam);
	}
	return FALSE;
}

}
}